@@ -2,7 +2,7 @@ package App::cpm::Worker::Installer;
 use strict;
 use warnings;
 use utf8;
-our $VERSION = '0.213';
+our $VERSION = '0.214';
 
 use CPAN::DistnameInfo;
 use CPAN::Meta;
@@ -156,7 +156,8 @@ sub fetch {
             $clean->($uri);
             my $basename = basename $uri;
             if ($uri =~ s{^file://}{}) {
-                File::Copy::copy($uri, $basename);
+                File::Copy::copy($uri, $basename)
+                    or next FETCH;
                 $dir = $self->menlo->unpack($basename)
                     or next FETCH;
                 last FETCH;
